package es.us.eii.pfc.jcrbib.data;

import static es.us.eii.pfc.jcrbib.utils.ConstantsUtils.LOGGERNAME;

import java.util.SortedSet;
import java.util.TreeSet;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.apache.log4j.Logger;

import es.us.eii.pfc.jcrbib.domain.MyArticle;
import es.us.eii.pfc.jcrbib.domain.User;
import es.us.eii.pfc.jcrbib.domain.ex.ItemAlreadyExistsException;
import es.us.eii.pfc.jcrbib.domain.ex.ItemNotExistsException;
import es.us.eii.pfc.jcrbib.utils.NodeUtils;
import es.us.eii.pfc.jcrbib.utils.RepositoryUtils;
import es.us.eii.pfc.jcrbib.utils.SessionUtils;

/**
 * 
 * Implementacin para el almacenamiento con JCR de los usuarios
 * 
 * 
 * 
 */
public class JcrUserDAO implements IUserDAO {

	private static Logger log = Logger.getLogger(LOGGERNAME);

	protected static String MY_ARTICLES_NODE_NAME = MessagesDAO
			.getString("JcrUserDAO.myarticles_name_tag"); //$NON-NLS-1$

	protected static String USERS_NODE_NAME = MessagesDAO
			.getString("JcrUserDAO.users_name_tag"); //$NON-NLS-1$

	private static String USER_NOT_FOUND_EX_FIRST = MessagesDAO
			.getString("JcrUserDAO.exceptions_user_not_found_first"); //$NON-NLS-1$

	private static String USER_NOT_FOUND_EX_LAST = MessagesDAO
			.getString("JcrUserDAO.exceptions_user_not_found_last"); //$NON-NLS-1$

	private static String USER_ALREADY_EXISTS_EX_FIRST = MessagesDAO
			.getString("JcrUserDAO.exceptions_user_already_exists_first"); //$NON-NLS-1$

	private static String USER_ALREADY_EXISTS_EX_LAST = MessagesDAO
			.getString("JcrUserDAO.exceptions_user_already_exists_last"); //$NON-NLS-1$

	protected JcrUserDAO() {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.IUserDAO#addUser(es.us.eii.pfc.jcrbib.domain.User)
	 */
	public void addUser(User user) throws ItemAlreadyExistsException {
		// Obtenemos una sesin para trabajar
		Session s = SessionUtils.getSession();
		Node init;
		try {
			log.debug(MessagesDAO
					.getString("JcrUserDAO.message_inserting_user")); //$NON-NLS-1$

			// Obtenemos el nodo inicial
			init = RepositoryUtils.getInitNode(s);
			Node users = init.getNode(USERS_NODE_NAME);
			String encNodeName = NodeUtils.encodeUserName(user.getName());

			// Si no existe el usuario
			if (!users.hasNode(encNodeName)) {

				// vamos creando la estructura en el repositorio

				// nodo del usuario
				Node newUserNode = users.addNode(encNodeName);

				// nodo de artculos
				newUserNode.addNode(MY_ARTICLES_NODE_NAME);

				// TODO: Si se aaden ms atributos, como el correo, colocarlos
				// aqui
				newUserNode.setProperty("pass", user.getPassword());

				// Aadimos los artculso
				SortedSet<MyArticle> myArticles = user.getMyArticles();
				JcrMyArticleDAO myArticleDAO = new JcrMyArticleDAO();
				for (MyArticle myAux : myArticles) {
					myArticleDAO.addMyArticle(myAux, user.getName(), s);
				}
			} else {
				// En otro caso elevamos una excepcin
				throw new ItemAlreadyExistsException(
						USER_ALREADY_EXISTS_EX_FIRST + user.getName()
								+ USER_ALREADY_EXISTS_EX_LAST);
			}
			// salvamos los cambios
			s.save();
			log.debug(MessagesDAO
					.getString("JcrUserDAO.message_succesfully_inserted_user")); //$NON-NLS-1$
		} catch (PathNotFoundException e) {
			log.error(e);
		} catch (RepositoryException e) {
			log.error(e);
		} finally {
			// y cerramos sesin
			s.logout();
			log.debug("Cerrando session");
		}
	}

	/* (non-Javadoc)
	 * @see es.us.eii.pfc.jcrbib.data.IUserDAO#getUser(java.lang.String)
	 */
	public User getUser(String name) throws ItemNotExistsException {
		Session s = SessionUtils.getSession();
		User user = null;
		user = getUser(name, s);
		s.logout();
		log.debug("Cerrando session");
		return user;
	}

	/* (non-Javadoc) Mtodo que obtiene un usuario
	 * @see es.us.eii.pfc.jcrbib.data.IUserDAO#getUser(java.lang.String)
	 */
	protected User getUser(String name, Session s)
			throws ItemNotExistsException {
		User outUser = null;
		Node init;
		try {

			init = RepositoryUtils.getInitNode(s);
			Node users = init.getNode(USERS_NODE_NAME);

			String encNameNode = NodeUtils.encodeUserName(name);
			
			// Si existe el usuario
			if (users.hasNode(encNameNode)) {
				// Obtenemos la informacin del usuario
				Node userNode = users.getNode(encNameNode);
				//TODO: Si hay ms atributos, obtenerlos aqu
				String encPassword = userNode.getProperty("pass").getString();
				JcrMyArticleDAO myArticleDAO = new JcrMyArticleDAO();
				outUser = new User(name, encPassword, false);
				outUser.setMyArticles(myArticleDAO.getMyArticles(name, s));
			} else {
				// En otro caso, elevamos una excepcin
				throw new ItemNotExistsException(USER_NOT_FOUND_EX_FIRST + name
						+ USER_NOT_FOUND_EX_LAST);
			}

		} catch (PathNotFoundException e) {
			log.error(e);
		} catch (RepositoryException e) {
			log.error(e);
		}
		return outUser;
	}

	/* (non-Javadoc)
	 * @see es.us.eii.pfc.jcrbib.data.IUserDAO#getUsers()
	 */
	public SortedSet<User> getUsers() {
		// Obtenemos la sesin
		Session s = SessionUtils.getSession();
		// Conjunto dnde almacenaremos los usuarios
		SortedSet<User> users = new TreeSet<User>();
		Node init;
		try {
			init = RepositoryUtils.getInitNode(s);
			Node usersNode = init.getNode(USERS_NODE_NAME);
			NodeIterator nt = usersNode.getNodes();
			String userName;
			// Recorremos todos los usuarios
			while (nt.hasNext()) {
				userName = NodeUtils.decodeUserName(nt.nextNode()
						.getName());
				try {
					users.add(getUser(userName, s));
				} catch (ItemNotExistsException e) {
					// En este catch no debera entrar
					log.error(e);
				}
			}
		} catch (PathNotFoundException e) {
			log.error(e);
		} catch (RepositoryException e) {
			log.error(e);
		} finally {
			s.logout();
			log.debug("Cerrando session");
		}
		return users;
	}

	/* (non-Javadoc)
	 * @see es.us.eii.pfc.jcrbib.data.IUserDAO#removeUser(java.lang.String)
	 */
	public User removeUser(String name) throws ItemNotExistsException {
		Session s = SessionUtils.getSession();
		User user = null;
		Node init;
		try {
			init = RepositoryUtils.getInitNode(s);
			Node users = init.getNode(USERS_NODE_NAME);
			user = getUser(name);

			// Primero eliminamos mis articulos
			JcrMyArticleDAO myArticleDAO = new JcrMyArticleDAO();
			myArticleDAO.removeMyArticles(name, s);

			// Y luego eliminamos el nodo
			String encUserNodeName = NodeUtils.encodeUserName(name);
			users.getNode(encUserNodeName).remove();
		} catch (PathNotFoundException e) {
			log.error(e);
		} catch (RepositoryException e) {
			log.error(e);
		} finally {
			s.logout();
			log.debug("Cerrando session");

		}
		return user;
	}

	/* (non-Javadoc)
	 * @see es.us.eii.pfc.jcrbib.data.IUserDAO#updateUser(es.us.eii.pfc.jcrbib.domain.User)
	 */
	public User updateUser(User user) throws ItemNotExistsException {
		Session s = SessionUtils.getSession();
		Node init;
		try {
			init = RepositoryUtils.getInitNode(s);
			Node users = init.getNode(USERS_NODE_NAME);
			String encNodeName = NodeUtils.encodeUserName(user.getName());

			if (users.hasNode(encNodeName)) {
				// TODO: Revisar las propiedades de primer nivel del usuario
				Node userNode = users.getNode(encNodeName);
				userNode.setProperty("pass", user.getPassword());

				// Establecemos ahora los articulos del usuario
				JcrMyArticleDAO myArticleDAO = new JcrMyArticleDAO();
				myArticleDAO.setMyArticles(user.getMyArticles(),
						user.getName(), s);
			} else {
				throw new ItemNotExistsException(USER_NOT_FOUND_EX_FIRST
						+ user.getName() + USER_NOT_FOUND_EX_LAST);
			}
			s.save();
		} catch (PathNotFoundException e) {
			log.error(e);
		} catch (RepositoryException e) {
			log.error(e);
		} finally {
			s.logout();
			log.debug("Cerrando session");

		}
		return user;
	}

	/* (non-Javadoc)
	 * @see es.us.eii.pfc.jcrbib.data.IUserDAO#existsUser(java.lang.String)
	 */
	public boolean existsUser(String userName) {
		boolean existsUser = false;
		try {
			existsUser = (getUser(userName) != null);
		} catch (Exception e) {
			// Deliberadamente vacio. Si hay errores es que no existe el usuario
		}
		return existsUser;
	}
}
