package es.us.eii.pfc.jcrbib.domain;

import java.util.SortedSet;
import java.util.TreeSet;

import es.us.eii.pfc.jcrbib.domain.ex.ItemAlreadyExistsException;
import es.us.eii.pfc.jcrbib.utils.SHA1BASE64;

/**
 * 
 * Clase que representa a los usuarios del sistema
 * 
 *
 */
public class User implements Comparable {
	private String name;

	private String password;

	private SortedSet<MyArticle> myArticles;

	public User(String name, String password, SortedSet<MyArticle> myArticles) {
		this.name = name;
		this.password = SHA1BASE64.encriptar(password);
		this.myArticles = myArticles;
	}

	public User(String name, String password, boolean encPassword) {
		this.name = name;
		if(encPassword) {
			this.password = SHA1BASE64.encriptar(password);
		} else {
			this.password = password;
		}
		this.myArticles = new TreeSet<MyArticle>();
	}

	public String getName() {
		// Creamos un nuevo string para evitar efectos colaterales
		return new String(this.name);
	}

	public SortedSet<MyArticle> getMyArticles() {
		return this.myArticles;
	}

	public void setMyArticles(SortedSet<MyArticle> myArticlesSortedSet) {
		this.myArticles = myArticlesSortedSet;
	}

	public void addMyArticle(MyArticle myArticle)
			throws ItemAlreadyExistsException {
		if (!this.myArticles.contains(myArticle)) {
			this.myArticles.add(myArticle);
		} else {
			throw new ItemAlreadyExistsException("MyArticle with id:"
					+ myArticle.getId() + " already exists");
		}
	}

	public boolean removeMyArticle(MyArticle myArticle) {
		return this.myArticles.remove(myArticle);
	}

	public boolean equals(User user) {
		return this.name.equals(user.getName());
	}

	public int compareTo(Object o) {
		return this.name.compareTo(((User) o).getName());
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = SHA1BASE64.encriptar(password);
	}

}
