package es.us.eii.pfc.jcrbib.utils;

import java.util.StringTokenizer;

/**
 * Clase de utilidad para poder trabajar con nodos (y sobre todo con sus
 * nombres)
 * 
 * Codifica y decodifica los nombres en la entrada/salida de informacin del
 * repositorio. Diseada para hacer que los nombres y ttulos puedan ser
 * almacenados como nombres de los nodos y a su vez, que estos cumplan las
 * restricciones sobre nombres de nodo
 * 
 * 
 * 
 */
public class NodeUtils {

	private static String SEPARATOR = "-";

	/**
	 * 
	 * Codifica un nombre de usuario
	 * 
	 * @param userName
	 *            Nombre de usuario a codificar
	 * @return El nombre codificado
	 */
	public static String encodeUserName(String userName) {
		StringBuffer sb = new StringBuffer();
		String codedChar;
		for (int i = 0; i < userName.length(); i++) {
			codedChar = Integer.toHexString((int) userName.charAt(i));
			sb.append(codedChar);
			if (i < userName.length() - 1) {
				sb.append(SEPARATOR);
			}
		}
		return sb.toString();
	}

	/**
	 * Mtodo que decodifica un nombre de usuario
	 * 
	 * @param encUserName
	 *            Nombde de usuario codificado
	 * @return El nombre de usuario decodificado
	 */
	public static String decodeUserName(String encUserName) {
		StringBuffer sb = new StringBuffer();
		StringTokenizer st = new StringTokenizer(encUserName, SEPARATOR);
		while (st.hasMoreTokens()) {
			sb.append((char) Integer.parseInt(st.nextToken(), 16));
		}
		return sb.toString();
	}

	/**
	 * Codifica el nombre de una categora
	 * 
	 * @param name Categora a codificar
	 * @return La categora codificada
	 */
	public static String encodeTagName(String name) {
		return encodeUserName(name.toLowerCase());
	}

	/**
	 * 
	 * Decodifica una categora
	 * @param encName Categora codificada
	 * @return La categora decodificada
	 */
	public static String decodeTagName(String encName) {
		return decodeUserName(encName);
	}

	/**
	 * 
	 * Codifica un nombre de autor
	 * @param name Nombre del autor
	 * @return Nombre del autor codificado
	 */
	public static String encodeAuthorName(String name) {
		return encodeUserName(name);
	}

	/**
	 * Decodifica un nombre de autor
	 * @param encName Nombre del autor codificado
	 * @return El nombre del autor decodificado
	 */
	public static String decodeAuthorName(String encName) {
		return decodeUserName(encName);
	}

}
