package es.us.eii.pfc.jcrbib.utils;

import static es.us.eii.pfc.jcrbib.utils.ConstantsUtils.LOGGERNAME;

import java.util.Iterator;
import java.util.Map;
import java.util.SortedSet;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Property;
import javax.jcr.PropertyIterator;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspWriter;

import org.apache.log4j.Logger;

import es.us.eii.pfc.jcrbib.data.DAOFactory;
import es.us.eii.pfc.jcrbib.data.IMyArticleDAO;
import es.us.eii.pfc.jcrbib.data.IUserDAO;
import es.us.eii.pfc.jcrbib.domain.MyArticle;
import es.us.eii.pfc.jcrbib.domain.Tag;
import es.us.eii.pfc.jcrbib.domain.User;

/**
 * 
 * Clase de utilidad para la presentacin de contenidos
 * 
 * 
 * 
 */
public class ShowUtils {

	private static Logger log;
	static {
		log = Logger.getLogger(LOGGERNAME);
		log.info("Log activado en ShowUtils");
	}

	/* Rutas de ficheros usadas */
	private static String tagPath = "/jcrbib/portal/bib/tags/?tag=";

	private static String userTagPath = "/jcrbib/portal/bib/?tag=";

	private static String IMG_PATH = "/jcrbib/portal/img/";

	private static String LINK_IMG_PATH = IMG_PATH + "link.gif";

	private static String PRIORITY_IMG_PATH = IMG_PATH + "prioridad";

	private static String USERS_IMG_PATH = IMG_PATH + "usuarios.gif";

	private static String OTHER_USERS_IMG_PATH = IMG_PATH
			+ "otros_usuarios.gif";

	private static String AUTHORS_IMG_PATH = IMG_PATH + "autores.gif";

	private static String TAGS_IMG_PATH = IMG_PATH + "tags.gif";

	/**
	 * 
	 * Mtodo que devuelve el HTML de resumen de un artculo. La presentacin
	 * depende de si se pasa un usuario o no
	 * 
	 * @param art
	 *            Artculo a presentar
	 * @param userNameAct
	 *            Usuario que posee el artculo. Puede ser null
	 * @return El HTML que describe el artculo
	 */
	public static String verResumenArticulo(MyArticle art, String userNameAct) {
		StringBuffer sb = new StringBuffer();
		sb.append("<a href=\"" + art.getUrl() + "\" " + "target=\"_blank\" "
				+ "title=\"Ver el articulo (pagina nueva)\">");
		sb.append("<strong class=\"tituloArticulo\">");
		sb.append(art.getTitle());
		sb.append("</strong>");
		sb.append("</a> ");
		sb.append("<a href=\"" + art.getUrl() + "\" " + "target=\"_blank\" "
				+ "title=\"Ver el articulo (pagina nueva)\">");
		sb.append("<img src=\"" + LINK_IMG_PATH
				+ "\" alt=\"Ver el enlace en una pagina nueva\" />");
		sb.append("</a>");
		if (userNameAct != null) {
			sb.append(" | Prioridad: <img src=\"" + PRIORITY_IMG_PATH
					+ art.getPriority() + ".gif\" title=\"Prioridad "
					+ art.getPriority() + "/5 \" />");
		}
		sb.append("<br />");

		sb.append("<span class=\"fichaArticulo\">");
		SortedSet<String> autores = art.getAuthors();
		if (!autores.isEmpty()) {
			sb.append("<img src=\"" + AUTHORS_IMG_PATH
					+ "\" alt=\"Icono autor(es)\"> Escrito por ");
			Iterator<String> itAutores = autores.iterator();
			String authorName;
			while (itAutores.hasNext()) {
				authorName = itAutores.next();
				sb.append("<a href=\"http://www.google.es/search?hl=es&q=%22"
						+ authorName.replace(' ', '+')
						+ "%22\" target=\"_blank\""
						+ " title=\"Mas informacion sobre " + authorName
						+ " en Internet \" >");
				sb.append(authorName);
				sb.append("</a>");
				if (itAutores.hasNext()) {
					sb.append(", ");
				} else {
					sb.append(". <br />");
				}
			}
		}
		SortedSet<String> tags;
		if (userNameAct != null) {
			tags = art.getTags();
		} else {
			IMyArticleDAO myArticleDAO = DAOFactory.getDAOFactory(
					DAOFactory.JCR).getMyArticleDAO();
			tags = myArticleDAO.getAllMyArticleTags(art.getId());
		}

		if (!tags.isEmpty()) {
			sb.append("<img src=\"" + TAGS_IMG_PATH
					+ "\" alt=\"Icono tag(s)\"> Marcado en ");
			Iterator<String> itTags = tags.iterator();
			while (itTags.hasNext()) {
				sb.append(tagUnsizedLink(itTags.next()));
				if (itTags.hasNext()) {
					sb.append(", ");
				} else {
					sb.append(". <br />");
				}
			}
		}

		return sb.toString() + verUsuariosArticulo(art, userNameAct)
				+ "</span>";
	}

	/*
	 * Devuelve la presentacin de usuarios de un artculo. Si userNameAct es
	 * null, los devuelve todos. En otro caso, devuelve el resto de usuario (si
	 * es que hay ms)
	 */
	private static String verUsuariosArticulo(MyArticle art, String userNameAct) {
		StringBuffer sb = new StringBuffer();
		DAOFactory factory = DAOFactory.getDAOFactory(DAOFactory.JCR);
		IUserDAO userDAO = factory.getUserDAO();

		SortedSet<User> usuarios = userDAO.getUsers();
		Iterator<User> it = usuarios.iterator();
		User u;
		boolean printedIntro = false;

		while (it.hasNext()) {
			u = it.next();
			// Si ya estamos en el usuario, no lo mostramos
			if (userNameAct != null && u.getName().equals(userNameAct)) {
				break;
			}
			if (u.getMyArticles().contains(art)) {
				if (!printedIntro) {
					if (userNameAct == null) {
						sb
								.append("<img src=\""
										+ USERS_IMG_PATH
										+ "\" alt=\"Icono autor(es)\"> En la biblioteca de ");
					} else {
						sb
								.append("<img src=\""
										+ OTHER_USERS_IMG_PATH
										+ "\" alt=\"Icono autor(es)\"> Otros usuarios que lo enlazaron: ");
					}
					printedIntro = true;
				}
				sb.append(" <a href=\"/jcrbib/portal/bib/?user=" + u.getName()
						+ "\">" + u.getName() + "</a>");
				sb.append(",");
			}
		}

		String out = sb.toString();
		if (out.length() > 2) {
			out = out.substring(0, out.length() - 1) + ".";
		}

		return out;
	}

	/**
	 * 
	 * Devuelve un enlace sin tamao a una categora
	 * 
	 * @param tagName
	 *            Nombre de la categora
	 * @return El HTML de enlace a la categora
	 */
	public static String tagUnsizedLink(String tagName) {
		return "<a href=\"" + tagPath + tagName + "\">" + tagName + "</a>";
	}

	/**
	 * Devuelve el enlace a una categora con tamao. Si el usuario no es nulo,
	 * el enlace dirige a la "categora C del usuario U". En otro caso, el
	 * enlace dirige a la "Categora C del sistema"
	 * 
	 * @param t
	 *            Informacin de la categora
	 * @param u
	 *            Usuario actual. Puede ser nulo
	 * @return El HTML de enlace a la categora
	 */
	public static String tagSizedLink(Tag t, User u) {
		String out;
		if (u == null) {
			out = "<a title=\"Tag " + t.getName() + ". Relevancia:"
					+ t.getWeight() + "\"" + "href=\"" + tagPath + t.getName()
					+ "\" style=\"vertical-align:middle; font-size:"
					+ (70 + t.getWeight() * 20) + "%\">" + t.getName() + "</a>";
		} else {
			out = "<a title=\"Tag " + t.getName() + ". Relevancia:"
					+ t.getWeight() + "\"" + "href=\"" + userTagPath
					+ t.getName() + "&amp;user=" + u.getName()
					+ "\" style=\"vertical-align:middle; font-size:"
					+ (70 + t.getWeight() * 20) + "%\">" + t.getName() + "</a>";
		}
		return out;
	}

	/**
	 * 
	 * Mtodo que obtiene la URL actual para mostrarla al usuario
	 * 
	 * @param request
	 *            Objeto HttpServletRequest del cual obtenemos esta informacin
	 * @return La URL actual
	 */
	public static String getActualUrl(HttpServletRequest request) {
		StringBuffer url = new StringBuffer(request.getRequestURL().toString());
		Map params = request.getParameterMap();
		Iterator it = params.keySet().iterator();
		String paramName;
		if (it.hasNext()) {
			url.append("?");
			while (it.hasNext()) {
				paramName = (String) it.next();
				url.append(paramName + "=" + request.getParameter(paramName));
				if (it.hasNext()) {
					url.append("&amp;");
				}
			}
		}
		return url.toString();
	}

	/*
	 * 
	 */
	/**
	 * Mtodo recursivo que sirve para representar la estructura del
	 * repositorio.
	 * 
	 * @param node
	 *            Nodo actual
	 * @param out
	 *            Buffer de salida
	 * @throws Exception
	 *             Si hay algn problema
	 */
	public static void showRepositoryAsList(Node node, JspWriter out)
			throws Exception {
		// Con esta condicin evitamos pintar la rama jcr:system, muy amplia y
		// con contenidos que no nos son de utilidad
		if (!node.getName().equals("jcr:system")) {
			out.print("<li style=\"list-style:none\">");
			NodeIterator it = node.getNodes();
			if (it.hasNext()) {
				out.print("+ ");
			} else {
				out.print("- ");
			}
			out.println(node.getName());
			PropertyIterator propIt = node.getProperties();
			Property prop;
			while (propIt.hasNext()) {
				prop = propIt.nextProperty();
				if (!prop.getName().equals("jcr:primaryType")) {
					out.print(" " + prop.getName() + "=\"");
					out.print(prop.getValue().getString());
					out.print("\"");
				}
			}
			out.print("<ul>");
			Node nl1;
			// Para cada nodo
			while (it.hasNext()) {
				nl1 = (Node) it.next();
				// Volvemos a invocar el mtodo
				showRepositoryAsList(nl1, out);
			}
			out.print("</ul>");
			out.println("</li>");
		}
	}
}